#!/usr/bin/perl -w
use strict;
use Math::Trig;

#     use File::stat;
#     $sb = stat($filename);
#     printf "File is %s, size is %s, perm %04o, mtime %s\n",
#         $filename, $sb->size, $sb->mode & 07777,
#         scalar localtime $sb->mtime;

my @minerals;
$minerals[0] = "0600_Anhydrite";
$minerals[1] = "0600_Barite";
$minerals[2] = "0600_Chalcopyrite";
$minerals[3] = "0600_Pyrite";
$minerals[4] = "0600_Pyrrhotite";
$minerals[5] = "0600_Sphalerite";
$minerals[6] = "0600_Wurtzite";
$minerals[7] = "0100_White";
$minerals[8] = "0300_Water";

my $PI = acos(-1);
my $rads;
my ($xdisp, $ydisp);
my ($dist, $zdisp) = (20, 0);
my ($anglename,$offset);

my $path = "/home/kristina/work/KM-exploration/oldRender/hari-ray-31";
chdir $path;
system("rm -f $path/logs/*");

my $datapath = "/home/kristina/work/KM-exploration/oldRender/datasets/archive";
my %datalist;
foreach my $volfile (glob($datapath."/*.vol")) {
   my $dataname;
   my @datainfo;
   ($dataname) = ($volfile =~ m|.*/([^\./]*)\.[^\.]*\.[^\.]*.vol|g);
   # data info is (full_filename, dim1, dim2, dim3, pos1, pos2, pos3)
   @datainfo = ($volfile, $volfile =~ m/[^\.]*\.([^x]*)x([^x]*)x([^x]*)\.([^_]*)_([^_]*)_([^_]*)\.vol/g);
   $datalist{$dataname} = \@datainfo;
}

my @lightnames; my @lightpositions; my @lightfoci;
$lightnames[0] = "top";
$lightnames[1] = "farleft";
$lightnames[2] = "farright";
$lightnames[3] = "left";
$lightnames[4] = "right";

my $datalink = "current.data";
my $lightlist = "autolightlist";

my $control = "control.cfg";
open(CONTROL, $control) or die "Can't open $control\n";
my %params;
while (my $l = <CONTROL>) {
	chomp($l);
	$l =~ s/\s*//g;
	my ($param, $values) = split(/=/, $l);
	$values =~ s/[{};]//g;
	my @values = split(/,/, $values);
	$params{$param} = \@values;
}
close CONTROL;

foreach my $d (@{$params{'data'}}) {
foreach my $c (@{$params{'conversion'}}) {
foreach my $s (@{$params{'scale'}}) {
foreach my $rw (@{$params{'rw'}}) {

my $enh = 1;
$enh = $1 if ($d =~ m/_enhx([\d]*)/g);
$s /= $enh;

my $header=
"#ifndef PARAMS_H
#define PARAMS_H

#include <cstdlib>
using namespace std;

#include \"../hari-ray-common/KM-spectra/costal.h\"

// acoustic to optical: old value = 57200, new value = 20-300
const float conversion = $c;

const double* const light = C0100_White;
const double* const absp[] = {K0300_Water,
                              K0600_Wurtzite}; // Not being used
const double* const scat[] = {S0300_Water, // Not being used
                              S0600_Wurtzite}; // Not being used
const float reflFactorW = $rw;

const float EPSILON = 0.003;
const double density_min = 0.0000000001;
const float scalefactor = $s; // voxel separation (meters)

// const float c = 0.85; // H-G phase function bias
// const float innerfactor = 0.469; // inner integral of H-G function
// const float outerfactor = 0.257; // outer integral of H-G function

// const float c = 0.9; // H-G phase function bias
// const float innerfactor = 0.63526;
// const float outerfactor = 0.21770;

const float c = 0.95; // H-G phase function bias
const float innerfactor = 0.861152;
const float outerfactor = 0.097372;

// const unsigned int colorLength = 3;
// const int colorInd[colorLength] = {28, 14, 7};
const unsigned int colorLength = 31;
const int colorInd[colorLength] = { 0,
                                    1, 2, 3, 4, 5, 6, 7, 8, 9,10,
                                   11,12,13,14,15,16,17,18,19,20,
                                   21,22,23,24,25,26,27,28,29,30};

#define DEBUG 5

#define X_POS 1
#define X_NEG 2
#define Y_POS 3
#define Y_NEG 4
#define Z_POS 5
#define Z_NEG 6

#endif // PARAMS_H";

my $hname = "params.h";
(chdir $path) or die "Failed to enter source directory\n";
open(HEADERFILE, ">".$hname) or die "Can't open $hname";
print HEADERFILE $header;
close HEADERFILE;

$rads = $PI / 2;
$xdisp = $dist * sin($rads);
$ydisp = $dist * (1 - cos($rads));

my @datainfo = @{$datalist{$d}};
my $dimStr = $datainfo[1]." ".$datainfo[2]." ".$datainfo[3];
my @lightcenter = ($datainfo[4]+$dist, $datainfo[5]+$dist, $datainfo[6]);
my $posStr = $lightcenter[0]." ".$lightcenter[1]." ".$lightcenter[2];
$lightpositions[0] = $lightcenter[0].' '.$lightcenter[1].' '.($lightcenter[2]+3);
$lightfoci[0] = (1-$lightcenter[0]).' '.$lightcenter[1].' '.($lightcenter[2]+2);
# $lightpositions[1] = ($lightcenter[0]-10).' '.$lightcenter[1].' '.($lightcenter[2]-3);
# $lightfoci[1] = ($lightcenter[0]-8).' '.(1-$lightcenter[1]).' '.($lightcenter[2]-2);
# $lightpositions[2] = ($lightcenter[0]+10).' '.$lightcenter[1].' '.($lightcenter[2]-3);
# $lightfoci[2] = ($lightcenter[0]+8).' '.(1-$lightcenter[1]).' '.($lightcenter[2]-2);
# $lightpositions[3] = ($lightcenter[0]-2).' '.$lightcenter[1].' '.($lightcenter[2]-2);
# $lightfoci[3] = ($lightcenter[0]-1).' '.(1-$lightcenter[1]).' '.($lightcenter[2]-1);
# $lightpositions[4] = ($lightcenter[0]+2).' '.$lightcenter[1].' '.($lightcenter[2]-2);
# $lightfoci[4] = ($lightcenter[0]+1).' '.(1-$lightcenter[1]).' '.($lightcenter[2]-1);

my $numlights = scalar @{$params{'lights'}};

if (!(scalar @ARGV)) {
(chdir $path."/..") or die "Failed to enter parent directory\n";
print "Making shadowVol... ";
(system("make shadow31 >> $path/logs/make.log") == 0) or die "Compilation failed.\n";
(chdir "datasets") or die "Failed to enter data directory\n";

print "Updating data link...\n";
if (-l $datalink) { (unlink $datalink) or die "Failed to remove old data link\n"; }
elsif (-e $datalink) { die "$datalink exists, but is not a link"; }
(symlink($datainfo[0],$datalink)) or die "Failed to create data link\n";

print "Generating... ";
open(LIGHTLIST, ">".$lightlist) or die "Can't open $lightlist";
print LIGHTLIST $numlights."\n";
foreach my $i (@{$params{'lights'}}) {
   print $lightnames[$i]."... ";
   (system("$path/shadowVol $datalink " .
           $lightnames[$i] . ".data  $dimStr " .
           $lightpositions[$i] . ' ' . $lightfoci[$i] .
           " >> $path/logs/shadow.log") == 0)
      or die "Failed to create the ".$lightnames[$i]." shadow volume.\n";
   print LIGHTLIST ($lightnames[$i].".data ".$lightpositions[$i]."\n");
}
close LIGHTLIST;
print "\n";
}

foreach my $m (@{$params{'mineral'}}) {
foreach my $am (@{$params{'ambient'}}) {
foreach my $df (@{$params{'diffuse'}}) {
foreach my $rm (@{$params{'rm'}}) {
foreach my $cs (@{$params{'colorscale'}}) {
foreach my $bg (@{$params{'bg'}}) {
foreach my $angle (@{$params{'angle'}}) {

(chdir $path) or die "Failed to enter source directory\n";

my $renderheader =
"#ifndef RENDERPARAMS_H
#define RENDERPARAMS_H

/* renderparams.h */
#include \"params.h\"

const int enh = $enh;

const double* const refl[] = {R0300_Water,
                              $m};

const float albedo = 0.85; // Not being used
const float ambient = $am;
const float diffuse = $df;
const float reflFactorM = $rm; // replaces albedo?

const float colorscale = $cs /* *31.0/colorLength*/;

const double* const bg = $bg;

#endif // RENDERPARAMS_H";
$hname = "renderparams.h";
open(HEADERFILE, ">".$hname) or die "Can't open $hname";
print HEADERFILE $renderheader;
close HEADERFILE;

my $mName;
($mName) = ($m =~ m/.*_([^_]*)/g);
my $bgName;
($bgName) = ($bg =~ m/.*_([^_]*)/g);

$rads = $angle * $PI / 180;
$xdisp = $dist * sin($rads);
$ydisp = $dist * (1 - cos($rads));

# two-phase (tp)
# source dataset
#    (some data names end in _1, or something similar, which may look confusing)
# number of wavelengths (at this time, always will be 31 or 3)
# number of lights (L)
# data conversion factor (c)
# scale of voxels in meters (ms)
# "mineral" data used for particle reflections (min)  // currently render-only
# ambient factor (a)  // currently render-only
# diffuse factor (d)  // currently render-only
# percentage of light the minerals reflect (rm)  // currently render-only
# percentage of light the water reflects (rw)  // currently render-only
# scaling factor applied to final accumulated color (cs)  // render-only
# bg color (bg)  // render-only
# angle of view from light (0 degrees if omitted)

$c = sprintf("%06d",$c);
$s = sprintf("%.2f",$s);
$rm = sprintf("%.3f",$rm);
$rw = sprintf("%.3f",$rw);
if ($angle > 0) {
  $anglename = sprintf("_%03d",$angle);
  $offset = "-o $xdisp $ydisp $zdisp ";
} else {
  $anglename = "";
  $offset = "";
}
my $name = "images/fromScript/tp_".$d."_31_L".$numlights."_c".$c."_ms".$s.
           "_min".$m."_a".$am."_d".$df."_rm".$rm."_rw".$rw."_cs".$cs."_bg$bgName".$anglename."_sidelit";
print $name."\n";

(chdir "..") or die "Failed to enter parent directory\n";

print "Making volRen... ";
(system("make render31 >> $path/logs/make.log") == 0) or die "Compilation failed.\n";
(chdir "datasets") or die "Failed to enter data directory\n";

print "Rendering... ";
(system("$path/volRen $datalink $lightlist $name.ppm " .
        "-s $dimStr -x 300 -y 300 -l $posStr " . $offset . "-t " .
        ">> $path/logs/render.log") == 0)
    or die "Failed to render the image\n";

print "Converting...\n";
(system("pnmtopng $name.ppm > $name.png") == 0) or die "Failed to convert ppm to png\n";
(unlink "$name.ppm") or die "Failed to delete the ppm\n";
}}}}}}} # inner loops (render only)
}}}} # outer loops (generate shadow volumes)

