/* ============================================================ */
/* File : costal_conv.c --- a set of common subroutines for 	*/
/*	colour conversion.					*/
/* Author: M. Chen						*/
/* Dates: 20 June 2004						*/
/* ============================================================ */

#include "costal.h"

#include <iostream>
using std::cerr;
#include <cmath>
#include <cstdlib>

/* ---------------------------------- */
/* 1964 SPD31 (400-700:10) to CIE XYZ */
/* ---------------------------------- */

vector<float> SPD31_to_XYZ(const vector<float>& SPD)
{
  vector<float> XYZ(3,0.0);
//   float N=0;
//   if (SPD.size() != 31) {
//    cerr << "Error: SPD vector did not have length 31, returning 0s\n";
//   }
//   else {
    for (int j=0; j<31; j++) {
      for (int i=0; i<3; i++) {
        XYZ[i] += SPD[j]*M_400_700_10[j][i+1];
//         if (XYZ[i] < 0.0) {
//           cerr << "Negative XYZ result!";
//           exit(-40);
//         }
      }
//       N += C0100_White[j]*M_400_700_10[j][2]; // Reference illuminant
    }
//     for (int i=0; i<3; i++) {
//       XYZ[i] /= N;
//     }
//   }
  return XYZ;
}

/* -------------- */
/* CIE XYZ to RGB */
/* -------------- */

// Convert using the method from
// http://www.brucelindbloom.com/index.html?Eqn_XYZ_to_RGB.html
vector<float> XYZ_to_RGB_BL(const vector<float>& XYZ)
{
  vector<float> RGB(3,0.0);
//   if (XYZ.size() != 3) {
//     cerr << "Error: XYZ vector did not have length 3, returning 0s\n";
//   }
//   else {
    for (int i=0; i<3; i++) {
      for (int j=0; j<3; j++) {
        RGB[i] += XYZ[j]*M_XYZ2SRGB[j][i];
      }
//       if (RGB[i] < 0.0) {
//         cerr << "Negative RGB result!";
//         exit(-40);
//       }
      RGB[i] = (RGB[i] > 0.0031308) ?
                 1.055*pow(RGB[i],1.0/2.4)-0.055 :
//                  12.92*((RGB[i] > 0.0) ? RGB[i] : 0.0);
                 12.92*RGB[i];
//       RGB[i] = (RGB[i] < 1.0) ? RGB[i] : 1.0;
    }
//   }
  return RGB;
}

/* -------------- */
/* CIE XYZ to RGB */
/* -------------- */

// XYZ_to_RGB(X, Y, Z, R, G, B)
// double	X, Y, Z;
// double	*R, *G, *B;
// {
// 	*R = X * M_XYZ2RGB[0][0] + Y * M_XYZ2RGB[1][0] + Z * M_XYZ2RGB[2][0];
// 	*G = X * M_XYZ2RGB[0][1] + Y * M_XYZ2RGB[1][1] + Z * M_XYZ2RGB[2][1];
// 	*B = X * M_XYZ2RGB[0][2] + Y * M_XYZ2RGB[1][2] + Z * M_XYZ2RGB[2][2];
// }
//
// /* -------------- */
// /* RGB to CIE XYZ */
// /* -------------- */
//
// RGB_to_XYZ(R, G, B, X, Y, Z)
// double	R, G, B;
// double	*X, *Y, *Z;
// {
// 	*X = R * M_RGB2XYZ[0][0] + G * M_RGB2XYZ[1][0] + B * M_RGB2XYZ[2][0];
// 	*Y = R * M_RGB2XYZ[0][1] + G * M_RGB2XYZ[1][1] + B * M_RGB2XYZ[2][1];
// 	*Z = R * M_RGB2XYZ[0][2] + G * M_RGB2XYZ[1][2] + B * M_RGB2XYZ[2][2];
// }
//
// /* ------------------ */
// /* CIE XYZ to CIE xyz */
// /* ------------------ */
//
// XYZ_to_xyz(X, Y, Z, x, y, z)
// double	X, Y, Z;
// double	*x, *y, *z;
// {
// 	double	sum;
//
// 	sum = X + Y + Z;
// 	if (IsZero(sum))
// 		*x = *y = *z = 0.0;
// 	else {
// 		*x = X / sum;
// 		*y = Y / sum;
// 		*z = Z / sum;
// 	}
// }
//
// /* ---------------------------------- */
// /* 1964 SPD31 (400-700:10) to CIE XYZ */
// /* ---------------------------------- */
//
// SPD31_to_XYZ(SPD, X, Y, Z)
// double	SPD[31];
// double	*X, *Y, *Z;
// {
// 	int	i;
// 	double	sumX, sumY, sumZ;
//
// 	sumX = sumY = sumZ = 0;
// 	for (i=0; i<31; i++) {
// 		sumX += SPD[i] * M_400_700_10[i][1];
// 		sumY += SPD[i] * M_400_700_10[i][2];
// 		sumZ += SPD[i] * M_400_700_10[i][3];
// 	}
// 	*X = sumX;	*Y = sumY;	*Z = sumZ;
// }
//
// /* ---------------------------------- */
// /* 1931 SPD31 (400-700:10) to CIE XYZ */
// /* ---------------------------------- */
//
// SPD31_to_XYZ_1931(SPD, X, Y, Z)
// double	SPD[31];
// double	*X, *Y, *Z;
// {
// 	int	i;
// 	double	sumX, sumY, sumZ;
//
// 	sumX = sumY = sumZ = 0;
// 	for (i=0; i<31; i++) {
// 		sumX += SPD[i] * M1931_400_700_10[i][1];
// 		sumY += SPD[i] * M1931_400_700_10[i][2];
// 		sumZ += SPD[i] * M1931_400_700_10[i][3];
// 	}
// 	*X = sumX;	*Y = sumY;	*Z = sumZ;
// }
