/* Volume Rendering by Ray Casting, using bounding spheres to do space
   leaping */

#include <cstdlib>
#include "rtCommon/art.h"

void readFile(int,char**);
rtColor rayCast(rtRay&,const rtPoint&);
rtColor getCumulative(rtRay&);
void getNormal(int,int,int,rtVector*);
void getNormal(int,int,int,int,rtVector*);
float getReflectivity(float,double);

float* volume;
vector<rtColor> shadow_volume;
int xsize,ysize,volx,voly,volz,volxvoly;
float f, theta=0, phi=0, psi=0, max_intensity;
rtPoint lightpos, eye, offset;

const int size = sizeof(float);

// float spacing = 0.1;
// float persp = 0.4;
float spacing = 0.2;
float persp = 0;
int colorLength = 3;
double visibility = 30;
rtColor bg(0.0, 0.0, 0.0);
rtPoint lowerlimit(0,0,0);
rtPoint upperlimit(200,120,90);
rtPoint lookat;

/******** MAIN ***********/

int main(int argc,char** argv) {

  char imageName[128];
  readFile(argc,argv);
  cout << "# of pixels: " << volx*volz << endl;

  ofstream out;
  for (int y=0; y<voly; y++) {
    snprintf(imageName, 128, "%s.y%03d.ppm", argv[6], y);
    out.open(imageName);
    if (!out) {
      cerr << "Could not create image file: " << imageName;
    } else {

      out << "P6" << endl;
      out << volx << " " << volz << endl;
      out << "255" << endl;
      int index = volxvoly*(volz-1) + y*volx;
//       cout << "index...";
      for (int z=volz-1; z>=0; z--) {
//         cout << index << "...";
        for(int x=0; x<volx; x++) {
          rtColor outColor(shadow_volume[index].getRGB());
          for (int channel=0; channel<3; channel++) {
            out.put(outColor.putChannel(channel));
          }
          index++;
        }
        index -= volx + volxvoly;
      }
//       cout << endl;
      out.close();
    }
  }

  return 0;
}

  /*************************
   ******* Functions *******
  *************************/

void readFile(int ac,char** av) {

  // Check whether there are enough arguments
  if (ac < 7)
  {
    cerr << "Usage: " << av[0] << "  shadowVol xDim yDim zDim colorLength imageBaseName\n";
    exit(1);
  }

  volx = atoi(av[2]);
  voly = atoi(av[3]);
  volz = atoi(av[4]);
  volxvoly = volx*voly;

  colorLength = atoi(av[5]);

    /******************************/
  /*   READ IN THE LIGHT DATA   */
  /******************************/

  // Assuming that the light volumes are the same size and color length
  // as the data volume, read in the light data
  long volume_size = volxvoly*volz;
  shadow_volume.reserve(volume_size);
  ifstream involume(av[1]);
  if (!involume) {
    cerr << "Unable to shadow volume file: " << av[1] << endl;
    exit(1);
  }
  else {
    cout << "Opened shadow volume " << av[1] << " with dimensions "
         << volx << "x" << voly << "x" << volz << endl;
  }
  int x=0, y=0, z=0;
  float rgb[colorLength];
  rtColor readcolor(colorLength,false);

  rtColor avgCol(colorLength, false);
  avgCol.setValues(0.0);
  // Since we know the eye and light positions,
  // precalculate and combine the light contributions at each voxel
  for(int i=0;i<volume_size;i++) {

    char* marker = reinterpret_cast<char*>(rgb);
    involume.read(marker, colorLength*sizeof(float));

    for (int channel=0; channel<colorLength; channel++) {
#ifdef BIGENDIAN_LIGHT
      for (j=0;j<size;j++) {
        temp[j] = marker[size - j - 1];
      }
      rgb[channel] = *(reinterpret_cast<float*>(temp));
#endif
      if (isnan(rgb[channel]) || isinf(rgb[channel])) {
        cerr << "Error reading shadow file at index " << i << endl;
        exit(-1);
      }
      readcolor.setChannel(channel, rgb[channel]);
    }
    shadow_volume.push_back(readcolor);
    readcolor *= 1.0/volume_size;
    avgCol += readcolor;
//     if (i%100 == 0) {
//       cout << "Color at index " << i << ": ";
//       shadow_volume[i].show();
//       cout << endl;
//     }

    if (x == volx-1) {
      x=0;
      if (y == voly-1) {
        y=0;
        z++;
      } else
        y++;
    }
    else
      x++;
  }
  cout << "avg color: " << avgCol << endl;

  involume.close();
}

