#ifndef ART_H
#define ART_H

#include "rtPoint.h"
#include "rtVector.h"
#include "rtColor.h"
#include <iostream>
#include <vector>
//#include <fstream>
//#include <cmath>
//using namespace std;

#define maxDepth 4


class rtRay  {
  protected:
    rtPoint origin;
    rtVector direction;
    rtColor color;
    rtColor absorption;
    float alpha;
  public:
    rtRay(const rtPoint& pt1=rtPoint(), const rtPoint& pt2=rtPoint(), int len=3) :
      origin(pt1),
      direction(rtVector(pt1,pt2)),
      color(rtColor(len)),
      absorption(rtColor(len))
    {
      Normalize();
      resetColors();
    }
    rtRay(const rtPoint& pt, const rtVector& vec, int len=3) :
      origin(pt),
      direction(vec),
      color(rtColor(len)),
      absorption(rtColor(len))
    {
      Normalize();
      resetColors();
    }

    void SetLocation(const rtPoint&, const rtPoint&);
    void SetLocation(const rtPoint&, const rtVector&);
    void resetColors();
    void setColor(float,float,float);
    void setColor(const vector<float>&);
    void setColor(const rtColor&);
    void setColorChannel(int, float);
    void addColor(const rtColor&);
    void setAbs(float,float,float);
    void setAbs(const vector<float>&);
    void setAbs(const rtColor&);
    void multAbs(const rtColor&);
    void setAlpha(float);
    void addAlpha(float);
    float getR() const;
    float getG() const;
    float getB() const;
    float getColorChannel(int) const;
    float getAR() const;
    float getAG() const;
    float getAB() const;
    float getAbsChannel(int) const;
    float getAlpha() const;
    float getX() const;
    float getY() const;
    float getZ() const;
    float getI() const;
    float getJ() const;
    float getK() const;
    void Normalize();
    const rtColor& showColor() const;
    const rtColor& showAbs() const;
    const rtVector& getDirection() const;
    const rtColor& getColor() const;
    const rtColor& getAbs() const;
};

class rtSphere
{
 protected:
    rtPoint centre;
    float radius;
    rtColor color;
    rtColor diffuseColor;
    rtColor specularColor;
    int specCoeff;
    float diffuse;
    float specular;
    float refraction;
    float transparency;
    float IoR;

 public:
    rtSphere()
	{}

    float getRefraction();
    rtPoint getCentre();
    float getIoR();
    float getDiffuse();
    rtColor getDiffuseColor();
    float getSpecular();
    rtColor getSpecularColor();
    float getSpecCoeff();
    rtColor getColor();
    float getTransparency();
    void setCentre(rtPoint);
    void setRadius(float);
    void setColor(rtColor);
    void setDiffuseColor(rtColor);
    void setSpecularColor(rtColor);
    void setSpecCoeff(int);
    void setDiffuse(float);
    void setSpecular(float);
    void setRefraction(float);
    void setTransparency(float);
    void setIoR(float);

    rtPoint intersect(rtRay,rtPoint);
    rtPoint inter(rtRay, rtPoint);
    rtVector findNormal(rtPoint);
};

class rtLight
{
 protected:
    rtPoint pt;
    rtColor col;

 public:
    rtLight() {}
    rtLight(rtPoint p,rtColor c) : pt(p),col(c) {}

    void setLocation(rtPoint);
    void setColor(rtColor);
    const rtPoint& getLocation() const;
    const rtColor& getColor() const;

    void setClamping(bool clamp, float minVal=0, float maxVal=1);
    bool getClamping() const;
    float getMin() const;
    float getMax() const;
};

#endif // ART_H
