#include "rtColor.h"
#include "KM-spectra/costal.h"
#include <cmath>

void rtColor::Set(float rr, float gg, float bb) {

  if (getLength() != 3)
    c.resize(3);

  setChannel(0, rr);
  setChannel(1, gg);
  setChannel(2, bb);

}
void rtColor::Set(const rtColor& col) {
  Set(col.getValues());
}
void rtColor::Set(const vector<float>& v) {
  int size = v.size();
  if (getLength() != v.size())
    c.resize(size);
  vector<float>::iterator i;
  vector<float>::const_iterator j;
  for (i=c.begin(),j=v.begin(); i!=c.end(); i++,j++) {
    *i = doClamp ? clamp(*j) : *j;
  }
//   for (int i=0; i<size; i++) {
//     c[i] = v[i];
//   }
}

void rtColor::setR(float rr) {
  setChannel(0, rr);
}
void rtColor::setG(float gg) {
  setChannel(1, gg);
}
void rtColor::setB(float bb) {
  setChannel(2, bb);
}

void rtColor::setValues(float val) {
  c.assign(getLength(), val);
}

void rtColor::setChannel(unsigned int i, float val) {
//   if (i < getLength()) {
    if (doClamp)
      val = clamp(val);
    c[i]=val;
//   } else {
//     cerr << "Warning: Color channel " << i << " is out of range (max. "
//         << getLength() << "), ignoring setChannel\n";
//   }
}

float rtColor::getR() const { return getChannel(0); }
float rtColor::getG() const { return getChannel(1); }
float rtColor::getB() const { return getChannel(2); }
float rtColor::getChannel(unsigned int i) const {
//   if (i < getLength())
    return c[i];
//   else {
//     cerr << "Warning: Color channel " << i << " is out of range (max. "
//          << getLength() << "), getChannel will return " << nullValue << endl;
//   }
//   return nullValue;
}
unsigned int rtColor::getLength() const {
  return c.size();
}
const vector<float>& rtColor::getValues() const {
  return c;
}
rtColor rtColor::getXYZ() const {
  if (getLength() == 31) {
    return rtColor(SPD31_to_XYZ(c), false);
  }
  return rtColor(*this);
}
float rtColor::getY() const {
  if (getLength() == 31) {
    return SPD31_to_XYZ(c)[1];
  }
  return 0.0;
}
rtColor rtColor::getRGB() const {
  if (getLength() == 31) {
     return rtColor(XYZ_to_RGB_BL(SPD31_to_XYZ(c)), true);
   }
   else if (getLength() != 3) {
     cerr << "Error: Color size was not 3 or 31."
          << "Don't know how to convert to RGB! Returning 0s.\n";
     rtColor err(3);
     err.setValues(0.0);
     return err;
   }
   return rtColor(*this);
}

unsigned char rtColor::putR() const { return putChannel(0); }
unsigned char rtColor::putG() const { return putChannel(1); }
unsigned char rtColor::putB() const { return putChannel(2); }
unsigned char rtColor::putChannel(unsigned int i) const {
  if (i < getLength()) {
    float val = c[i];
    // If it's not already clamped, clamp it now
    if (!doClamp)
      val = clamp(val);
    return static_cast<unsigned char>(val*255+0.5);
  }
  else {
    cerr << "Warning: Color channel out of range, "
         << "putChannel will return 0\n";
  }
  return 0;
}



rtColor rtColor::operator * (const rtColor& col) const
{
  unsigned int size = getLength();
  rtColor result(size);
  if (size != col.getLength()) {
    result.setValues(nullValue);
    cerr << "Can't multiply, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      result.setChannel(i, c[i]*col.getChannel(i));
    }
  }
  return result;
}

rtColor rtColor::operator + (const rtColor& col) const
{
  unsigned int size = getLength();
  rtColor result(size);
  if (size != col.getLength()) {
    result.setValues(nullValue);
    cerr << "Can't add, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      result.setChannel(i, c[i]+col.getChannel(i));
    }
  }
  return result;
}

rtColor rtColor::operator - (const rtColor& col) const
{
  unsigned int size = getLength();
  rtColor result(size);
  if (size != col.getLength()) {
    result.setValues(nullValue);
    cerr << "Can't subtract, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      result.setChannel(i, c[i]-col.getChannel(i));
    }
  }
  return result;
}

const rtColor& rtColor::operator = (const rtColor& col)
{
  Set(col);
  return *this;
}

const rtColor& rtColor::operator *= (const rtColor& col) {

  unsigned int size = getLength();
  if (size != col.getLength()) {
    cerr << "Can't multiply, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      setChannel(i, c[i]*col.getChannel(i));
    }
  }
  return *this;
}

const rtColor& rtColor::operator += (const rtColor& col) {

  unsigned int size = getLength();
  if (size != col.getLength()) {
    cerr << "Can't add, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      setChannel(i, c[i]+col.getChannel(i));
    }
  }
  return *this;
}

const rtColor& rtColor::operator -= (const rtColor& col) {

  unsigned int size = getLength();
  if (size != col.getLength()) {
    cerr << "Can't subtract, colors have different length.\n";
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      setChannel(i, c[i]-col.getChannel(i));
    }
  }
  return *this;
}

bool rtColor::operator != (const rtColor& col) const
{
  if (getLength() != col.getLength())
    return true;
  for (unsigned int i=0; i<getLength(); i++) {
//     if (col.getChannel(i) < 0)
//       cerr << "operator!= is the culprit!!\n"; else
    if (c[i] != col.getChannel(i))
      return true;
  }
  return false;
}

bool rtColor::operator == (const rtColor& col) const
{
  return !(*this != col);
}

rtColor rtColor::Scale(float sc) const
{
  unsigned int size = getLength();
  bool err = false;

  rtColor result(size);
  if (isnan(sc)) {
    cerr << "Scale factor is NaN, returning null\n";
    err = true;
  }
  else if (isinf(sc)) {
    cerr << "Scale factor is Inf, returning null\n";
    err = true;
  }
  if (err) {
    result.setValues(nullValue);
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      result.setChannel(i, c[i]*sc);
    }
  }
  return result;
}
const rtColor& rtColor::operator *= (float sc) {
  unsigned int size = getLength();
  bool err = false;

  if (isnan(sc)) {
    cerr << "Scale factor is NaN, returning null\n";
    err = true;
  }
  else if (isinf(sc)) {
    cerr << "Scale factor is Inf, returning null\n";
    err = true;
  }
  if (err) {
    setValues(nullValue);
  }
  else {
    for (unsigned int i=0; i<size; i++) {
      setChannel(i, c[i]*sc);
    }
  }
  return *this;
}

float rtColor::clamp(float val) const {
  if (val>max)
    return max;
  if (val<min)
    return min;
  return val;
}

bool rtColor::isNull() const {
  vector<float>::const_iterator i;
  for (i=c.begin(); i!=c.end(); i++) {
    if (*i == nullValue)
      return true;
  }
  return false;
}

rtColor rtColor::Interpolate(const rtColor& aCol, const rtColor& bCol,
                             float pcent) {
  unsigned int size = aCol.getLength();
  rtColor result(size);
//   if (size != bCol.getLength()) {
//     cerr << "Can't interpolate, colors have different length.\n";
//     result.setValues(nullValue);
//   }
//   else {
    for (unsigned int i=0; i<size; i++) {
      float a = aCol.getChannel(i);
      float b = bCol.getChannel(i);
//       if (a < 0 || b < 0)
//         cerr << "Interpolate is the culprit!!\n"; else
      result.setChannel(i, a + (b-a)*pcent);
    }
//   }
  return result;
}
void rtColor::setInterpolate(const rtColor& b, float pcent) {
  unsigned int size = getLength();
  for (unsigned int i=0; i<size; i++) {
    c[i] += (b.getChannel(i)-c[i])*pcent;
  }
}

ostream& rtColor::show(ostream& stream) const {
  stream << "(";
  vector<float>::const_iterator i;
  for (i=c.begin(); i!=c.end()-1; i++) {
    stream << *i << ", ";
  }
  stream << *i << ")";
  return stream;
}
