#ifndef RTCOLOR_H
#define RTCOLOR_H

#include "rtBase.h"
#include <iostream>
#include <vector>

class rtColor : public rtBase {
  protected:
    static const float nullValue = -1;
    vector<float> c;
    bool doClamp;
    float min, max;
    float clamp(float) const;

  public:
 rtColor(int numChannels=3, bool clamp=true) : rtBase(),
      doClamp(clamp), min(0), max(1)
    {
      c.resize(numChannels);
    }
 rtColor(const rtColor& col, bool clamp=true) : rtBase(),
      doClamp(clamp), min(0), max(1)
    {
      Set(col);
    }
 rtColor(const vector<float>& init, bool clamp=true) : rtBase(),
      doClamp(clamp), min(0), max(1)
    {
      Set(init);
    }
 rtColor(float red, float green, float blue, bool clamp=true) : rtBase(),
      doClamp(clamp), min(0), max(1)
    {
      Set(red, green, blue);
    }

    void setClamping(bool clamp, float minVal=0, float maxVal=1) {
      doClamp = clamp;
      min = minVal;
      max = maxVal;
    }
    bool getClamping() const { return doClamp; }
    float getMin() const { return min; }
    float getMax() const { return max; }

    void Set(float, float, float);
    void Set(const rtColor&);
    void Set(const vector<float>&);
    void setR(float);
    void setG(float);
    void setB(float);
    void setChannel(unsigned int, float);
    void setValues(float val);
    float getR() const;
    float getG() const;
    float getB() const;
    float getChannel(unsigned int) const;
    unsigned int getLength() const;
    const vector<float>& getValues() const;
    rtColor getXYZ() const;
    float getY() const;
    rtColor getRGB() const;
    unsigned char putR() const;
    unsigned char putG() const;
    unsigned char putB() const;
    unsigned char putChannel(unsigned int) const;
    rtColor operator * (const rtColor& col) const;
    rtColor operator + (const rtColor& col) const;
    rtColor operator - (const rtColor& col) const;
    const rtColor& operator = (const rtColor& col);
    const rtColor& operator *= (const rtColor& col);
    const rtColor& operator += (const rtColor& col);
    const rtColor& operator -= (const rtColor& col);
    bool operator != (const rtColor& col) const;
    bool operator == (const rtColor& col) const;
    rtColor Scale(float sc) const;
    const rtColor& operator *= (float sc);
    bool isNull() const;

    static rtColor Interpolate(const rtColor&, const rtColor&, float);
    void setInterpolate(const rtColor&, float);

    virtual ostream& show(ostream&) const;
};

#endif
