#include "rtPoint.h"
#include <math.h>
using namespace std;

void rtPoint::Set(const rtPoint& p) {
  Set(p.getX(),p.getY(),p.getZ());
}

void rtPoint::Set(float xCo, float yCo, float zCo) {
  x=xCo; y=yCo; z=zCo;
}

float rtPoint::getX() const { return x; }
float rtPoint::getY() const { return y; }
float rtPoint::getZ() const { return z; }
float rtPoint::getComponent(int comp) const {
  if (comp == 0) return x;
  if (comp == 1) return y;
  return z;
}

void rtPoint::setX(float xc) {x = xc;}
void rtPoint::setY(float yc) {y = yc;}
void rtPoint::setZ(float zc) {z = zc;}

float rtPoint::getDistance(const rtPoint& far) const {
  float xdist = far.getX() - x;
  float ydist = far.getY() - y;
  float zdist = far.getZ() - z;
  return sqrt(xdist*xdist + ydist*ydist + zdist*zdist);
}

float rtPoint::getRadius() const {
  return sqrt(x*x+y*y+z*z);
}

void rtPoint::Rotate(float th,float phi,float psi) {

  float x1,y1,z1;
  float x2,y2,z2;

  // Rotate about x axis first.

  x1 = x;
  y1 = y*cos(th) - z*sin(th);
  z1 = y*sin(th) + z*cos(th);

  // Now Rotate about y axis.

  x2 = x1*cos(phi) + z1*sin(phi);
  y2 = y1;
  z2 = z1*cos(phi) - x1*sin(phi);

  // Now Rotate about z axis.

  x = x2*cos(psi) + y2*sin(psi);
  y = y2*cos(psi) - x2*sin(psi);
  z = z2;
  /*
  double t,p,r;
  t = th + atan(x/z);
  p = phi + atan(y/z);

  r = sqrt(x*x+y*y+z*z);

  //cout << "theta " << t << " phi " << p << " rad " << r << endl;

  x = r*cos(p)*sin(t);
  z = r*cos(p)*cos(t);
  y = r*sin(p);
  */
}


bool rtPoint::operator < (const rtPoint& pt) const {
  if (z<pt.z)
    return true;
  else
    return false;
}


ostream& rtPoint::show(ostream& stream) const {
  return stream << "(" << x << " " << y << " " << z << ")";
}

istream& operator>>(istream& stream, rtPoint& pt) {
  float x, y, z;
  char c = 0;

  stream >> c;
  if (c == '(') {
    stream >> x >> c;
    while (c == ',' || c == ' ') { stream >> c; }
    stream.putback(c);  // first non-, non-space character
    stream >> y >> c;
    while (c == ',' || c == ' ') { stream >> c; }
    stream.putback(c);
    stream >> z >> c;
    while (c == ' ') { stream >> c; }
    if (c != ')') { stream.putback(c); stream.clear(ios_base::badbit); }
  }
  if (stream) { pt.Set(x, y, z); }
  return stream;
}
