#include "rtVector.h"
#include <math.h>
using namespace std;

void rtVector::Set(rtPoint p1, rtPoint p2) {
  i = p2.getX() - p1.getX();
  j = p2.getY() - p1.getY();
  k = p2.getZ() - p1.getZ();
}

void rtVector::Set(rtVector vec) {
  i = vec.getI();
  j = vec.getJ();
  k = vec.getK();
}

float rtVector::getI() const { return i; }
float rtVector::getJ() const { return j; }
float rtVector::getK() const { return k; }
float rtVector::getComponent(int comp) const {
  if (comp == 0) return i;
  if (comp == 1) return j;
  return k;
};

void rtVector::setI(float x) { i = x; }
void rtVector::setJ(float y) { j = y; }
void rtVector::setK(float z) { k = z; }

float rtVector::getMagnitude() const { return sqrt(i*i + j*j + k*k); }

rtVector rtVector::Scale(float sc) const {
  return rtVector(i*sc,j*sc,k*sc);
}

void rtVector::Normalize() {
  float mag = getMagnitude();
  if (mag)
    Set(Scale(1.0/mag));
//   else
//     cerr << "Warning: Vector magnitude is 0, unable to normalize\n";
}

rtVector rtVector::operator - (rtVector v) const {
  float ix = i - v.getI();
  float jy = j - v.getJ();
  float kz = k - v.getK();

  return rtVector(ix,jy,kz);
}


float rtVector::dot(rtVector vec) const {
  return (i*vec.i + j*vec.j + k*vec.k);
}

rtVector rtVector::cross(rtVector v) const {
  float newI, newJ, newK;
  newI = j*v.getK() - k*v.getJ();
  newJ = k*v.getI() - i*v.getK();
  newK = i*v.getJ() - j*v.getI();
  return rtVector(newI, newJ, newK);
}

rtVector rtVector::cross(rtVector v1, rtVector v2) {
  float newI, newJ, newK;
  newI = v1.getJ()*v2.getK() - v1.getK()*v2.getJ();
  newJ = v1.getK()*v2.getI() - v1.getI()*v2.getK();
  newK = v1.getI()*v2.getJ() - v1.getJ()*v2.getI();
  return rtVector(newI, newJ, newK);
}

ostream& rtVector::show(ostream& stream) const {
  return stream  << i << "i + " << j << "j + " << k << "k ";
}
