#include "utilities/config.h"
#include "rtCommon/rtPoint.h"
#include "rtCommon/rtLightSpec.h"
#include <iostream>
#include <sstream>
#include <string>
using namespace std;


Config::Config(istream& in) {
  exitFlag = false;
  string temp;
  ostringstream record;
  bool done = false;
  while (in >> temp && !done) {  // next token
    record << temp << " ";
    if (temp == "dataFile" || temp == "DataFile") {
      in >> dataFileName;
      record << dataFileName << " ";
    } else if (temp == "dimensions" || temp == "Dimensions") {
      in >> x >> y >> z;
      record << x << " " << y << " " << z << " ";
    } else if (temp == "light" || temp == "Light") {
      rtLightSpec tempLight;
      in >> tempLight;
      lightV.push_back(tempLight);
      record << tempLight << " ";
    } else if (temp == "exit" || temp == "Exit") {
      exitFlag = true;  // meaning configured processes should exit
      done = true;
    } else if (temp == "done" || temp == "Done") {
      done = true;
    } else {
      // ignore it.
    }
  }
  
  buff = record.str();

}

const char* Config::error() const {
  ostringstream errbuf;
  if (dataFileName == "") {
    errbuf << "Missing dataFile. ";
  }
  if (x <= 0 || y <= 0 || z <= 0) {
    errbuf << "Incorrect dimensions. ";
  }
  if (lightV.size() < 1) {
    errbuf << "No light information. ";
  } 

  string errstr = errbuf.str();
  if (errstr == "") {
    return 0;
  } else {
    return errstr.c_str(); 
  }
}

bool Config::exit() const {
  return exitFlag;
}

int Config::volx() const {
  return x;
}

int Config::voly() const {
  return y;
}

int Config::volz() const {
  return z;
}

const vector<rtLightSpec>& Config::lights() const {
  return lightV;
}

const string& Config::dataFile() const {
  return dataFileName;
}

void Config::show(ostream& stream) const {
  stream << endl
	 << "DataFile " << dataFileName << endl
	 << "dimensions " << x << " " << y << " " << z << endl;
  const vector<rtLightSpec> vect = lights();
  vector<rtLightSpec>::const_iterator i;
  for (i = vect.begin(); i < vect.end(); i++) {
    stream << "light " << *i << endl;
  }
}

DDebug& operator<<(DDebug& dstream, const Config& c) {
  ostream& s = dstream.stream();
  c.show(s);;
  return dstream;
}

ostream& operator<<(ostream& s, const Config& c) {
  c.show(s);
  return s;
}
