#ifndef SV_CONFIG_H
#define SV_CONFIG_H

//
// This reads a file and extracts some configuration variables for
// shadowVol.  It is just meant to isolate this code, not to be a 
// classy object.  More of a struct with a function.
//
// Note that this parses configs, but does not read or write the 
// data files.
//
// DataFile <path>
// ImageFile <path>
// dimensions <int int int>
// light <point> <path>
// observer <point>
// conversion <float>
// scaleFactor <float>
// reflFactor <float>
// done
//
//  The order of elements doesn't matter.  Elements are separated by 
// whitespace so putting them one per line is readable.
//  Paths can be absolute or relative to the directly where the application
// is run.
//  There can be multiple 'light' lines.
//  See configuration/config.example as an example.
//  
//  Not that the 'done' line marks the data above as complete and starts
// the processing.   It will stop processing.  Of course, if you are just
// putting one set of parameters through a process, it isn't really 
// necessary.  When a Config object writes itself back to a stream it
// does not write the 'done' tag.
// 


#include "rtCommon/rtPoint.h"
#include "rtCommon/rtLightSpec.h"
#include "utilities/ddebug.h"
#include <iostream>
#include <vector>
#include <string>

class Config {
 protected:
  int x, y, z;
  vector<rtLightSpec> lightV;
  string dataFileName;
  string buff;
  bool exitFlag;

 public:
  Config(istream& );

  // Files.  data is input data.  shadow in output & input. image is output
  const string& dataFile() const;
  
  // volume dimensions
  int volx() const;
  int voly() const;
  int volz() const;

  // data for image processing.
  const vector<rtLightSpec>& lights() const;

  // Did we detect an error in construction
  virtual const char* error() const;

  // Did we get told to exit
  bool exit() const;

  virtual void show(ostream&) const;
  friend ostream& operator<<(ostream&, const Config&) ;

};

DDebug& operator<<(DDebug&, const Config&);  
ostream& operator<<(ostream&, const Config&);

#endif
