#include "vrConfig.h"
#include <string>
#include <sstream>
#include <istream>

vrConfig::vrConfig(istream& in) : Config(in) {
  // Now parse the info recorded by our parent for vr specific tags.
  istringstream buffer(buff);
  string temp;
  while (buffer >> temp) {
    if (temp == "image" || temp == "Image") {
      buffer >> ix >> iy;
    } else if (temp == "imageFile" || temp == "ImageFile") {
      buffer >> imageFileName;
    } else if (temp == "lightCenter" || temp == "LightCenter") {
      buffer >> light;
    } else if (temp == "conversion" || temp == "Conversion") {
      buffer >> conversionF;
    } else if (temp == "reflFactorW" || temp == "ReflFactorW") {
      buffer >> reflFactorWv;
    } else if (temp == "reflFactorM" || temp == "ReflFactorM") {
      buffer >> reflFactorMv;
    } else if (temp == "scaleFactor" || temp == "ScaleFactor") {
      buffer >> scale;
    } else if (temp == "enh" || temp == "Enh") {
      buffer >> enhv;
    } else if (temp == "ambient" || temp == "Ambient" ) {
      buffer >> ambientv;
    } else if (temp == "diffuse" || temp == "Diffuse") {
      buffer >> diffusev;
    } else if (temp == "colorscale" || temp == "Colorscale") {
      buffer >> colorscalev;
    } 
    //    if (temp == "bg" || temp == "Bg") {
    //      buffer >> bgv;
    //    }
  }
}

const char* vrConfig::error() const {
  ostringstream errbuf;
  errbuf << Config::error();

  if (imageFileName == "") {
    errbuf << "No imageFile information. ";
  }
  if (ix <= 0 || iy <= 0) {
    errbuf << "No image size information. ";
  }
  if (conversionF <= 0) {
    errbuf << "Incorrect conversion. ";
  }
  if (scale <= 0) {
    errbuf << "Incorrect scaleFactor. ";
  } 
  if (reflFactorWv <= 0) {
    errbuf << "Incorrect reflFactorW. ";
  }
  if (reflFactorMv <= 0) {
    errbuf << "Incorrect reflFactorM. ";
  }
  // accept that light may be (0,0,0)
  // enhv, ambientv, diffusev, colorscalev & bgv should all be checked, but I 
  // don't know if they should all be positive.

  string errstr = errbuf.str();
  if (errstr == "") {
    return 0;
  } else { 
    return errstr.c_str();
  }
}


const string& vrConfig::imageFile() const {
  return imageFileName;
}

unsigned int vrConfig::imageX() const {
  return ix;
}

unsigned int vrConfig::imageY() const {
  return iy;
}

const rtPoint& vrConfig::lightCenter() const {
  return light;
}

float vrConfig::conversion() const {
  return conversionF;
}

float vrConfig::reflFactorW() const {
  return reflFactorWv;
}

float vrConfig::reflFactorM() const {
  return reflFactorMv;
}

float vrConfig::scaleFactor() const {
  return scale;
}

int vrConfig::enh() const {
  return enhv;
}

float vrConfig::ambient() const {
  return ambientv;
}

float vrConfig::diffuse() const {
  return diffusev;
}

float vrConfig::colorscale() const {
  return colorscalev;
}

//const double* vrConfig::bg() const {
//  return bgv;
//}

void vrConfig::show(ostream& stream) const {
  stream << static_cast<Config>(*this)
	 << "ImageFile " << imageFileName << endl
	 << "Image " << ix << " " << iy << endl
	 << "LightCenter " << light << endl
	 << "conversion " << conversionF << endl
	 << "reflFactorW " << reflFactorWv << endl 
	 << "reflFactorM " << reflFactorMv << endl 
	 << "scaleFactor " << scale << endl
	 << "enh " << enhv << endl
	 << "ambient " << ambientv << endl
	 << "diffuse " << diffusev << endl
	 << "colorscale " << colorscalev << endl
	 << "bg " << bgv << endl
    ;
}
